/*
 * This program is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License, version 2.1 as published by the Free Software
 * Foundation.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, you can obtain a copy at http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * or from the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * Copyright (c) 2012 Pentaho Corporation..  All rights reserved.
 */
 
pen.require(["viz-Zoom/viz-util"], function(){

    pentaho.viz.util.RegisterVisualization( {
        vizId: 'Zoom',
        pluginId: 'viz-Zoom'
    } );

/*
    Visualization Metadata
    These objects describe the visualizations provided by this library.
*/
pentaho.visualizations.push({
    id: 'Zoom',                          // unique identifier
    type: 'line',                       // generic type id
    source: 'protovis',                          // id of the source library
    name: 'Zoom',                     // visible name, this will come from a properties file eventually
    'class': 'pentaho.viz.zoom',          // type of the Javascript object to instantiate
    needsColorGradient: false,
    customType: '',
    maxValues: [50000,50000,50000,50000],
    getDropZoneLabel: function(type){
      var label = type;
      return label;
    },
    args: {
    },
    propMap: [
    ],
    helper: {
        canRefreshReport: function(report) {
          var ok = true;
          if (report.findGemsByGembarId("rows").length == 0 || report.findGemsByGembarId("measures").length == 0)
            ok = false;
          return ok;
        }
    },
    dataReqs: [                             // dataReqs describes the data requirements of this visualization
        {
            name: 'Default',
            reqs: 
                [
                    {   id: 'rows',             // id of the data element
                        dataType: 'string',         // data type - 'string', 'number', 'date', 'boolean', 'any' or a comma separated list
                        dataStructure: 'column',    // 'column' or 'row' - only 'column' supported so far
                        caption: 'rows',        // visible name
                        required: true,              // true or false
                        allowMultiple: true         // true or false
                    },
                    {   id: 'measures',
                        dataType: 'number',
                        dataStructure: 'column',
                        caption: 'measures',
                        required: true,
                        allowMultiple: false         // true or false
                    },
                    {
                        id: 'fixedyaxis',
                        dataType: 'boolean',
                        ui: {
                          label: "fixedyaxis",
                          group: "options",
                          type: 'checkbox'
                        }
                    },
                    {
                      id: "optionsBtn",
                      dataType: 'none',
                      ui: {
                          group: "options",
                          type: "button",
                          label: "chartoptions"
                      }
                    }
                ]
        }
    ]
});

} );

pen.require(["cdf/lib/CCC/protovis",
    "cdf/lib/CCC/protovis-msie","viz-SunBurst/viz-util"], function(pv){
    
/*
    visualization constructor
    This takes an HTML DOM element as a parameter
*/
pentaho.viz.zoom = function( element ) {
    this.debug = false;
    this.element = element;
    this.elementName = element.id;
    this.cdaTable = null;
    this.dataTable = null;
    this.vizOptions = null;
    this.fixedyaxis = true;
    this.labelColor = "#000000";
    this.paletteNo = 0;
    this.version = "1.0";
        
    this.init();
}

pentaho.viz.zoom.prototype.resize = function() {
    this.draw(this.dataTable, this.vizOptions);
}

/*
    draw()
    
    dataTable   a pentaho.DataTable object with the data to display
    vizOptions  the options for the visualization
*/
pentaho.viz.zoom.prototype.draw = function( pdataTable, options ) {

    this.fixedyaxis = false;
    if( this.debug ) console.log('pentaho.viz.zoom.draw()');

        if( options ) {
            if( options.fixedyaxis ) {
                this.fixedyaxis = options.fixedyaxis;
            }
        }
        
        // check the controller current visualizations arguments (set by the properties panel)
        if( this.controller && this.controller.currentViz && this.controller.currentViz.args ) {
            var fixedyaxis = this.controller.currentViz.args.fixedyaxis;
            if( fixedyaxis ) {
                this.fixedyaxis = true;
            }
        }
        
        // check Analyzer's report document (yuck)
        if( cv && cv.rptEditor && cv.rptEditor.report && cv.rptEditor.report.reportDoc ) {
            var fixedyaxis = cv.rptEditor.report.reportDoc.getChartOption('fixedyaxis');
            if( fixedyaxis ) {
                this.fixedyaxis = fixedyaxis;
            }
        }

    // handle fonts, background
    pentaho.viz.util.HandleCommonOptions( this, options );

    // set up the color ranges
    pentaho.viz.util.setupColorRanges(this);

    // apply the background color/gradient
    pentaho.viz.util.ApplyBackground(this.element, options);

    this.vizOptions = options;
    this.dataTable = pdataTable;
    this.data = [];

    this.rowsCols = [];
    this.measureCol = -1;
    
    for( var colNo=0; colNo<this.dataTable.getNumberOfColumns(); colNo++) {
        var dataReq = this.dataTable.getColumnProperty(colNo,'dataReq');
        if(dataReq) {
            for (var idx=0; idx < dataReq.length; idx++) {
                if( dataReq[idx].id == 'rows' ) {
                    this.rowsCols.push( colNo );
                }
                else if( dataReq[idx].id == 'measures' ) {
                    this.measureCol = colNo;
                }
            }
        }
    }

    var min = null;
    var max = null;
    for( var rowNo=0; rowNo<this.dataTable.getNumberOfRows(); rowNo++ ) {
        // create the label
        var label = '';
        for( var colNo=0; colNo< this.rowsCols.length; colNo++ ) {
            if( colNo > 0 ) {
                label += '~';
            }
            label += this.dataTable.getFormattedValue(rowNo, colNo);
        }
        var item = {
            x: rowNo, 
            label: label, 
            y: this.dataTable.getValue(rowNo,this.measureCol ), 
            idx: rowNo,
            tooltip: label + "<br/>" + this.dataTable.getFormattedValue(rowNo,this.measureCol )
        };
        this.data.push(item)
        if( min == null ) {
            min = item.y;
            max = item.y;
        } else {
            min = Math.min( min, item.y );
            max = Math.max( max, item.y );
        }
    }

    if( this.debug ) console.log('pentaho.viz.zoom.draw creating scales');

    /* Scales and sizing. */
    
    var bottomChartHeight = 50;
    this.leftMargin = 80;
    var labelHeight = 100;
    var w = pentaho.viz.util.getClientWidth(this);
    var h = pentaho.viz.util.getClientHeight(this);
    var h1 = h - labelHeight*2 - bottomChartHeight;
    var h2 = bottomChartHeight;
    this.x = pv.Scale.linear(0,this.data.length-1).range(0, w - this.leftMargin -20);
    this.y = pv.Scale.linear(min, max).range(0, h2);

    /* Interaction state. Focus scales will have domain set on-render. */
    if( this.data.length < 100 ) {
        this.i = {x:0, dx: Math.min( this.data.length-1, 10 )};
    } else {
        this.i = {x:0, dx: Math.max( this.data.length/20, 20 ) };
    }
    this.fx = pv.Scale.linear().range(0, w);
    this.fy = pv.Scale.linear().range(0, h1);

    /* Root panel. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating root panel');
    var vis = new pv.Panel()
        .width(w)
        .height(h)
        .bottom(5)
        .left(5)
        .right(5)
        .top(5)
        .canvas(this.svgDiv)
        ;

    vis['viz'] =  this;

    if( this.debug ) console.log('pentaho.viz.zoom.draw creating focus panel');
    /* Focus panel (zoomed in). */
    var focus = vis.add(pv.Panel)
        .def("init", function() {
            var d1 = this.parent.viz.x.invert(this.parent.viz.i.x);
            var d2 = this.parent.viz.x.invert(this.parent.viz.i.x + this.parent.viz.i.dx);
            var dd = this.parent.viz.data.slice(
                    Math.max(0, pv.search.index(this.parent.viz.data, d1, function(d) { return d.x }) - 1),
                    pv.search.index(this.parent.viz.data, d2, function(d) { return d.x }) + 1);
            this.parent.viz.fx.domain(d1, d2);
            this.parent.viz.fy.domain(this.parent.viz.fixedyaxis ? this.parent.viz.y.domain() : [0, pv.max(dd, function(d) { return d.y })] );
            return dd;
          })
        .top(0)
        .width(w - this.leftMargin -20)
        .left(this.leftMargin)
        .height(h1);

    /* X-axis ticks. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating x-axis');
    focus.add(pv.Rule)
        .data(function() { return this.parent.parent.viz.fx.ticks( Math.min(this.parent.parent.viz.data.length-1,8)) })
        .left(this.fx)
        .strokeStyle("#eee")
        .bottom(0)
        .anchor("bottom")
        .add(pv.Label)
        .text(function(d) { var idx = Math.floor(d); if( !this.parent.parent.viz.data[idx] ) console.log(idx); return this.parent.parent.viz.data[idx].label; } )
        .textAlign("right")
        .textAngle(-Math.PI/4)
        .textStyle(this.labelColor)
        .font(this.labelFontStr)
        ;
            
    /* Y-axis ticks. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating y-axis');
    focus.add(pv.Rule)
        .data(function() { return this.parent.parent.viz.fy.ticks(7)})
        .bottom(this.fy)
        .strokeStyle(function(d) { return d ? "#aaa" : "#000" })
      .anchor("left").add(pv.Label)
        .text(this.fy.tickFormat)
        .textStyle(this.labelColor)
        .font(this.labelFontStr)
        ;

    /* Focus area chart. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating focus chart');
    focus.add(pv.Panel)
        .overflow("hidden")
      .add(pv.Area)
        .data(function() { return focus.init() })
        .left(function(d) { return this.parent.parent.parent.viz.fx(d.x) })
        .bottom(1)
        .height(function(d) { return this.parent.parent.parent.viz.fy(d.y) })
        .fillStyle(pentaho.palettes[this.paletteNo].colors[0])
        .title( function(){ return "";} ) // Prevent browser tooltip
      .anchor("top").add(pv.Line)
        .fillStyle(null)
        .strokeStyle(pentaho.palettes[this.paletteNo].colors[0])
        .lineWidth(2);

    /* Context panel (zoomed out). */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating context panel');
    var context = vis.add(pv.Panel)
        .top(h1+labelHeight)
        .width(w - this.leftMargin -20)
        .left(this.leftMargin)
        .height(h2);

    /* X-axis ticks. */
    context.add(pv.Rule)
        .data(this.x.ticks())
        .left(this.x)
        .strokeStyle("#eee")
        .bottom(0)
        .anchor("bottom")
        .add(pv.Label)
        .text(function(d) { var idx = Math.floor(d); if( !this.parent.parent.viz.data[idx] ) console.log(idx); return this.parent.parent.viz.data[idx].label; } )
        .textAlign("right")
        .textAngle(-Math.PI/4)
        .textStyle(this.labelColor)
        .font(this.labelFontStr)
        ;

    /* Y-axis ticks. */
    context.add(pv.Rule)
        .bottom(0);

    /* Context area chart. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating context chart');
    context.add(pv.Area)
        .data(this.data)
        .left(function(d) { return this.parent.parent.viz.x(d.x) } )
        .bottom(1)
        .height(function(d) { return this.parent.parent.viz.y(d.y) } )
        .fillStyle(pentaho.palettes[this.paletteNo].colors[0])
      .anchor("top").add(pv.Line)
        .strokeStyle(pentaho.palettes[this.paletteNo].colors[0])
        .lineWidth(2);

    /* The selectable, draggable focus region. */
    if( this.debug ) console.log('pentaho.viz.zoom.draw creating drag region');
    context.add(pv.Panel)
        .data([this.i])
        .cursor("crosshair")
        .events("all")
        .event("mousedown", pv.Behavior.select())
        .event("select", focus)
      .add(pv.Bar)
        .left(function(d) { return d.x })
        .width(function(d) { return d.dx })
        .fillStyle("rgba(255, 128, 128, .4)")
        .cursor("move")
        .event("mousedown", pv.Behavior.drag())
        .event("drag", focus);

    if( this.debug ) console.log('pentaho.viz.zoom.draw rendering');
    vis.render();

    if( this.debug ) console.log('pentaho.viz.zoom.draw() done');
        
}

pentaho.viz.zoom.prototype.init = function() {

    if( this.debug ) console.log('pentaho.viz.zoom.init()');
    this.svgDiv = document.createElement('DIV');
    this.element.appendChild( this.svgDiv );
    this.svgDiv.id = "svgDiv";
    this.svgDiv.style.position="absolute";
    this.svgDiv.style.top="0px";
    this.svgDiv.style.left="0px";
    this.svgDiv.style.width="100%";
    this.svgDiv.style.height="100%";   
    this.svgDiv.style.overflow="hidden";       
    this.svgDiv.style.textAlign="center";       
};

});

